__doc__ = """\
: AqGuard
:   20 15:19:57 MSK 2002
:    (c) 2002
c: $Id: libaltdist.py,v 1.4 2002/03/01 16:04:30 paul Exp $

:
        libaltdist.py

:
          distutils,   rpm
           AltLinux
"""

import os, sys
import glob
import string
import re
from distutils.core import Command, DEBUG
from distutils.extension import Extension
from distutils.file_util import write_file
from distutils.command.bdist_rpm import bdist_rpm
from distutils.errors import *

from types import *
from xreadlines import xreadlines

class bdist_altrpm(bdist_rpm):

    user_options = bdist_rpm.user_options + [
        ("prefix=", None, "Prefix for installing modules"), 
        ("lsummary=", None, "Summary for locale (first string is locale)"),
        ("ldescription=", None, "Description for locale (first string is locale)"),]

    def initialize_options(self):
        self.prefix = None
        self.lsummary = None
        self.ldescription = None
        bdist_rpm.initialize_options(self)

    def finalize_options(self):
        self.set_undefined_options('bdist_rpm', ('release', 'release'))
        self.set_undefined_options('bdist_rpm', ('serial', 'serial'))
        self.set_undefined_options('bdist_rpm', ('group', 'group'))
        self.set_undefined_options('bdist_rpm', ('icon', 'icon'))
        self.set_undefined_options('bdist_rpm', ('provides', 'provides'))
        self.set_undefined_options('bdist_rpm', ('requires', 'requires'))
        self.set_undefined_options('bdist_rpm', ('conflicts', 'conflicts'))
        self.set_undefined_options('bdist_rpm', ('build_requires', 'build_requires'))
        self.set_undefined_options('bdist_rpm', ('obsoletes', 'obsoletes'))
        self.set_undefined_options('bdist_rpm', ('doc_files', 'doc_files'))
        bdist_rpm.finalize_options(self)

    def finalize_package_data(self):
        fchangelog = open("CHANGELOG", "r")
        self.changelog = string.join(fchangelog.readlines(), '')
        fchangelog.close()
#       print 'Changelog before: ', self.changelog
        bdist_rpm.finalize_package_data(self)
        self.ensure_string('lsummary')
        self.ensure_string('ldescription')
#       print 'Changelog after: ', self.changelog

    def _make_spec_file(self):
        """Generate the text of an RPM spec file and return it as a
        list of strings (one per line).
        """
        spec_file = [
            'Name: ' + self.distribution.get_name(),
            'Version: ' + self.distribution.get_version(),
            'Release: ' + self.release,
            ]
        if self.serial is not None:
            spec_file.append('Serial: ' + self.serial)
            
        if self.prefix is not None:
            spec_file.append('Prefix: ' + self.prefix)

        spec_file.extend([
            '',
            'Summary: ' + self.distribution.get_description(),])
        if self.lsummary:
            (locale, summary) = string.split(self.lsummary, '\n')
            spec_file.append('Summary(%s): %s'% (locale, summary))

        # XXX yuck! this filename is available from the "sdist" command,
        # but only after it has run: and we create the spec file before
        # running "sdist", in case of --spec-only.
        if self.use_bzip2:
            spec_file.append('Source0: %{name}-%{version}.tar.bz2')
        else:
            spec_file.append('Source0: %{name}-%{version}.tar.gz')

        spec_file.extend([
            'Copyright: ' + self.distribution.get_licence(),
            'Group: ' + self.group, ])

        # noarch if no extension modules
        if not self.distribution.has_ext_modules():
            spec_file.append('BuildArchitectures: noarch')

        for field in (
#                      'Vendor',
                      'Packager',
                      'Provides',
                      'Requires',
                      'Conflicts',
                      'Obsoletes',
                      ):
            val = getattr(self, string.lower(field))
            if type(val) is ListType:
                spec_file.append('%s: %s' % (field, string.join(val)))
            elif val is not None:
                spec_file.append('%s: %s' % (field, val))

        if self.distribution.get_url() != 'UNKNOWN':
            spec_file.append('Url: ' + self.distribution.get_url())

#        if self.distribution_name:
#            spec_file.append('Distribution: ' + self.distribution_name)

        if self.build_requires:
            spec_file.append('BuildRequires: ' +
                             string.join(self.build_requires))

        if self.icon:
            spec_file.append('Icon: ' + os.path.basename(self.icon))

        spec_file.extend([
            '',
            '%description',
            self.distribution.get_long_description()
            ])
        if self.ldescription:
            delim = string.find(self.ldescription, '\n')
            locale = self.ldescription[:delim]
            description = self.ldescription[delim:]
            spec_file.append('%description -l ' +  locale)
            spec_file.extend(string.split(description, '\n'))

        # put locale descriptions into spec file
        # XXX again, suppressed because config file syntax doesn't
        # easily support this ;-(
        #for locale in self.descriptions.keys():
        #    spec_file.extend([
        #        '',
        #        '%description -l ' + locale,
        #        self.descriptions[locale],
        #        ])

        # rpm scripts
        # figure out default build script
        def_build = "%s setup.py build" % self.python
        if self.use_rpm_opt_flags:
            def_build = 'env CFLAGS="$RPM_OPT_FLAGS" ' + def_build

        # insert contents of files

        # XXX this is kind of misleading: user-supplied options are files
        # that we open and interpolate into the spec file, but the defaults
        # are just text that we drop in as-is.  Hmmm.

        script_options = [
            ('prep', 'prep_script', "%setup"),
            ('build', 'build_script', def_build),
            ('install', 'install_script',
             ("%s setup.py install "
              "--root=$RPM_BUILD_ROOT "
              "--record=INSTALLED_FILES") % self.python),
#            ('clean', 'clean_script', "rm -rf $RPM_BUILD_ROOT"),
            ('pre', 'pre_install', None),
            ('post', 'post_install', None),
            ('preun', 'pre_uninstall', None),
            ('postun', 'post_uninstall', None),
        ]

        for (rpm_opt, attr, default) in script_options:
            # Insert contents of file referred to, if no file is refered to
            # use 'default' as contents of script
            val = getattr(self, attr)
            if val or default:
                spec_file.extend([
                    '',
                    '%' + rpm_opt,])
                if val:
                    spec_file.extend(string.split(open(val, 'r').read(), '\n'))
                else:
                    spec_file.append(default)


        # files section
        spec_file.extend([
            '',
            '%files -f INSTALLED_FILES',
            '%defattr(-,root,root)',
            ])

        if self.doc_files:
            spec_file.append('%doc ' + string.join(self.doc_files))

        if self.changelog:
            spec_file.extend([
                '',
                '%changelog',])
            spec_file.extend(self.changelog)

        spec_path = os.path.join('build', self.distribution.get_name() + '.spec')
        self.execute(write_file,
                     (spec_path,
                      spec_file),
                     "writing '%s'" % spec_path)

        cleanup_spec = ['cleanup_spec',]
        cleanup_spec.append(spec_path)
# Add changelog parse spec file and not find icon file
#        cleanup_spec.append("-a '--define _topdir %s/%s'" % (os.getcwd(), self.rpm_base))
        self.spawn(cleanup_spec)

        fspec = open(spec_path, 'r')
        spec_file = fspec.readlines()
        spec_file = map(string.rstrip, spec_file)
        fspec.close()

        return spec_file

    # _make_spec_file ()

    def _format_changelog(self, changelog):
        """Format the changelog correctly and convert it to a list of strings
        """
        if not changelog:
            return changelog
        new_changelog = []
        for line in string.split(string.strip(changelog), '\n'):
            line = string.strip(line)
            if len(line) == 0:
                continue
            if line[0] == '*':
                new_changelog.extend(['', line])
            elif line[0] == '-':
                new_changelog.append(line)
            else:
                new_changelog.append('  ' + line)
                
        # strip trailing newline inserted by first changelog entry
        if not new_changelog[0]:
            del new_changelog[0]
        
        return new_changelog


class add_changelog(bdist_altrpm):
    def finalize_options(self):
        self.set_undefined_options('bdist_altrpm', ('prefix', 'prefix'))
        self.set_undefined_options('bdist_altrpm', ('lsummary', 'lsummary'))
        self.set_undefined_options('bdist_altrpm', ('ldescription', 'ldescription'))
        bdist_altrpm.finalize_options(self)

    def run (self):
        if DEBUG:
            print "before _get_package_data():"
            print "vendor =", self.vendor
            print "packager =", self.packager
            print "doc_files =", self.doc_files
            print "changelog =", self.changelog

        rpm_dir = {}
        for d in ('SOURCES', 'SPECS', 'BUILD', 'RPMS', 'SRPMS'):
            rpm_dir[d] = os.path.join(self.rpm_base, d)
            self.mkpath(rpm_dir[d])
        spec_dir = rpm_dir['SPECS']

        spec_path = os.path.join(spec_dir, "%s.spec" % self.distribution.get_name())
        self.execute(write_file,
                     (spec_path,
                      self._make_spec_file()),
                     "writing '%s'" % spec_path)
        if self.icon:
            if os.path.exists(self.icon):
                self.copy_file(self.icon, rpm_dir['SOURCES'])
            else:
                raise DistutilsFileError, \
                      "icon file '%s' does not exist" % self.icon

        add_changelog_cmd = ['add_changelog',]
        add_changelog_cmd.append(spec_path)
# Add changelog parse spec file and not find icon file
#        add_changelog_cmd.append("-a '--define _topdir %s/%s'" % (os.getcwd(), self.rpm_base))
        self.spawn(add_changelog_cmd)

        fspec = open(spec_path, 'r')
        fchangelog = open('CHANGELOG', 'w')
        is_log = 0
        for line in xreadlines(fspec):
            if is_log:
                fchangelog.write(line)
            if line.strip() == '%changelog':
                is_log = 1
        fchangelog.close()
        
def version(name):
#    print os.listdir(os.path.join(os.getcwd(), 'lib'))
    verf = open(name)
    ver = verf.readline()
    verr = re.compile('-([0-9]+\.[0-9]+\.[0-9].*)')
    verm = verr.search(ver)
    if verm:
        ver = verm.group(1)
    else:
        ver = '0.0.0'
    verf.close()
    return ver
